required_plugins = []

tree = {
    "storage": {"engine": "fs"},
    "network": {"x": 150.0, "y": 150.0, "z": 150.0},
    "partitions": {},
    "cell_types": {},
    "placement": {},
    "connectivity": {},
    "simulations": {},
}
